package de.intevation.mxd.reader;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.IFillSymbol;
import com.esri.arcgis.display.ILineSymbol;
import com.esri.arcgis.display.IMarkerSymbol;
import com.esri.arcgis.display.MarkerFillSymbol;
import com.esri.arcgis.display.esriSimpleFillStyle;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import org.w3c.dom.Element;

/**
 * Reads marker line symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class MarkerFillSymbolReader
extends AbstractSymbolReader{

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(MarkerFillSymbolReader.class);

    /**
     * Private member.
     */
    private MarkerFillSymbol symbol;

    public MarkerFillSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof MarkerFillSymbol) {
            this.symbol = (MarkerFillSymbol)symbol;
        }
        else {
            throw new Exception("Not a MarkerFillSymbol!");
        }
    }


    public MarkerFillSymbolReader(IFillSymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof MarkerFillSymbol) {
            this.symbol = (MarkerFillSymbol)symbol;
        }
        else {
            throw new Exception("Not a MarkerFillSymbol!");
        }
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read()
    throws Exception {
        logger.debug("read()");

        Element symbolElement = util.addSymbol(parent);

        symbolElement.setAttribute("name", symbol.getNameString());
        symbolElement.setAttribute("style", "fill");
        symbolElement.setAttribute(
            "xoffset",
            String.valueOf(symbol.getXOffset()));
        symbolElement.setAttribute(
            "yoffset",
            String.valueOf(symbol.getYOffset()));
        symbolElement.setAttribute(
            "xseparation",
            String.valueOf(symbol.getXSeparation()));
        symbolElement.setAttribute(
            "yseparation",
            String.valueOf(symbol.getYSeparation()));

        int style = symbol.getStyle();
        switch(style) {
            case esriSimpleFillStyle.esriSFSCross:
                symbolElement.setAttribute("fillstyle", "cross"); break;
            case esriSimpleFillStyle.esriSFSSolid:
                symbolElement.setAttribute("fillstyle", "solid"); break;
            case esriSimpleFillStyle.esriSFSVertical:
                symbolElement.setAttribute("fillstyle", "vertical"); break;
            case esriSimpleFillStyle.esriSFSHorizontal:
                symbolElement.setAttribute("fillstyle", "horizontal"); break;
            case esriSimpleFillStyle.esriSFSForwardDiagonal:
                symbolElement.setAttribute("fillstyle", "fwdiagonal"); break;
            case esriSimpleFillStyle.esriSFSBackwardDiagonal:
                symbolElement.setAttribute("fillstyle", "bwdiagonal"); break;
            case esriSimpleFillStyle.esriSFSDiagonalCross:
                symbolElement.setAttribute("fillstyle", "diagonalcross");
                break;
            default: symbolElement.setAttribute("fillstyle", "empty");
        }

        if(symbol.getColor() instanceof IRgbColor) {
            IRgbColor color = (IRgbColor)symbol.getColor();
            Color c = new Color (
                color.getRed(),
                color.getGreen(),
                color.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            symbolElement.setAttribute("transparency",
                String.valueOf(color.getTransparency()));
        }
        else {
            RgbColor col = new RgbColor();
            col.setRGB(symbol.getColor().getRGB());
            Color c = new Color (
                col.getRed(),
                col.getGreen(),
                col.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            symbolElement.setAttribute("transparency",
                String.valueOf(col.getTransparency()));
        }

        ILineSymbol ls = symbol.getOutline();
        LineSymbolReader lsr = new LineSymbolReader();
        if(lsr.canRead(ls)) {
            lsr.setSymbol(ls);
            lsr.setUtil(util);
            lsr.setParent(symbolElement);
            lsr.read();
        }
        else {
            logger.debug("The type of " + ls.getClass().toString() +
                         " is not implemented!");
        }

        IMarkerSymbol sym = symbol.getMarkerSymbol();
        MarkerSymbolReader msr = new MarkerSymbolReader();
        if(msr.canRead(sym)) {
            msr.setSymbol(sym);
            msr.setParent(symbolElement);
            msr.setUtil(util);
            msr.read();
        }
        else {
            logger.debug("The type of " + sym.getClass().toString() +
                         " is not implemented!");
        }
        return parent;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
